#!/bin/sh
#
# NAME
#  agentadmin - SAS Deployment Agent administrative script
#
# DESCRIPTION 
#  Starts, Stops and reports the status of a SAS Deployment Agent
#
# RETURN VALUES
#  0 - success (desired administrative task completed)
#  1 - Usage error or abnormal termination
#
#######################################################################
LANG=C
TRUE="1"                       # Boolean true
FALSE="0"                      # Boolean false
cmdname="`basename $0`"
agentdir="`dirname "$0"`"      # Agent installation directory
hostname="`uname -n`"          # hostname to be used in naming of pid file
pwd="`pwd`"                    # Current working directory
uid="`whoami`"                 # Current username
rc="1"                         # Return code of function calls
vm=""                          # Jave Virtual Machine (JRE) to use
pid=""                         # PID to kill
pidfile="$agentdir/server.${hostname}.pid" # name of the pid file

#
# Discover the PID at runtime of an agent.  Caller assumes
# responsibility for   
# 
agent_pid () {
   pid="`cat "$pidfile" 2>/dev/null`"
   return
}

#
# Read the agent's PID from pid file
#
read_pid () {
   if [ ! -r "$pidfile" ] ; then
      echo "$cmdname: error: agent PID file does not exist"
      exit 1
   fi
   
   agent_pid
   
   if [ -n "$pid" ] && [ $pid -gt 1 ] ; then
      return 
   else
      echo "$cmdname: error: PID ($pid) is an invalid process id"
      exit 1
   fi
}

#
# Set the Virtual Machine JRE/JDK to use
#  
set_vm () {
   input=$1

   if [ -d "$input" -a -f "$input/bin/java" ] ; then
      vm="$input/bin/java"
      return
   fi

   if [ -f "$input" -a "`basename "$input"`" = "java" ] ; then
      vm="$input"
      return
   fi
}

#
# Internal function used to handle evaluation and display of the results for 
# the stat function
# @param state - up or down
# @param status - status of kill -0
# @param -v (optional) - verbose flag
#
stat_result_handler () {
   if [ "$1" = "up" ] ; then
      if [ "$2" = "0" ] ; then
         result=$TRUE
         rc="0"
         if [ "$3" = "-v" ] ; then
            echo "$cmdname: agent status: SAS Deployment Agent is running. Process identifier (PID) is $pid"
         fi
      else
         result=$FALSE
         rc="1"
         if [ "$3" = "-v" ] ; then
            echo "$cmdname: agent status: SAS Deployment Agent is down."
         fi
      fi 
   else 
      if [ "$2" != "0" ] ; then
         rc="0"
         result=$TRUE
         if  [ "$3" = "-v" ] ; then
            echo "$cmdname: agent status: SAS Deployment Agent is down"
         fi
      else  
         result=$FALSE
         rc="1"
         if [ "$3" = "-v" ] ; then
            echo "$cmdname: agent status: SAS Deployment Agent is running. Process identifier (PID) is $pid"
         fi
      fi                
   fi    
}


#
# Check the status of a SAS Deployment Agent to determine if it is up
# or not.
# @param state (required) "up" or "down"
# @param -v (optional) verbose flag: when set echo status result to terminal
# @return TRUE if state of Deployment Agent matches value FALSE otherwise
#
agent_stat () {
   # Validate args
   if [ "$1" != "up" ] && [ "$1" != "down" ] ; then
      echo "$cmdname: agent status: unrecognized parameter ($1)"
      exit 1
   fi
   
   if [ "$2" != "" ] && [ "$2" != "-v" ] ; then
      echo "$cmdname: agent status: unrecognized parameter ($2)"
      exit 1
   fi
   
   state=$1
   if [ -n "$pid" ] && [ "$pid" -gt 1 ] ; then
      # kill -0 tests if the signal can be sent...
      kill -0 "$pid" 1>/dev/null 2>&1
      stat_result_handler "$state" "$?" "$2"
      return         
   fi
   
   # Okay we don't have a PID figure out what message to send
   if [ $state = "down" ] ; then
      stat_result_handler "$state" "0" "$2"
   fi
      
   if [ $state = "up" ] ; then
      stat_result_handler "$state" "1" "$2"
   fi    
}

#
# Stop an Agent if it's running
# 
agent_stop () {
   if [ "$1" != "" ] ; then
      echo "$cmdname: agent stop: unrecognized parameter ($1)"
      exit 1
   fi
   
   # Go fetch the PID
   read_pid
   
   # Error checking: Make sure the agent referenced in the PID file is running
   agent_stat "up"
   if [ $result = $FALSE ] ; then
      echo "$cmdname: agent stop:  SAS Deployment Agent with (PID) $pid is already down.  Removing PID file."
      # It is safe to remove the PID associated with this process
      rm "$pidfile"
      exit 1
   fi
  
   echo "$cmdname: agent stop: shutdown requested for local SAS Deployment Agent with Process Identifier (PID) $pid..."
   kill $pid 1>/dev/null 2>&1
   if [ $? = 0 ] ; then
      # The kill signal was successfully sent to the process set scripts rc to 0
      rc=0
      # Before we can output a message to the user to let them know the shutdown finished,
      # add a slight delay to give the shutdown and OS some time to complete.  
      sleep 1
      agent_stat "down"
      if [ $result = $TRUE ] ; then 
         # It is safe to remove the PID associated with this process
         rm "$pidfile"
         echo "$cmdname: agent stop: SAS Deployment Agent shutdown."
         exit 0
      else
         echo "$cmdname: agent stop: SAS Deployment Agent is shutting down.  Use $cmdname: stat down: to determine when it has finished"
         exit 0
      fi 
   else
      echo "$cmdname: agent stop: SAS Deployment Agent shutdown request failed"
      exit 1
   fi    
}

#
# Start a new SAS Deployment Agent when one isn't running
#
agent_start () {
   if [ "$1" != "" ] ; then
      echo "$cmdname: agent start: unrecognized parameter ($1)"
      exit 1
   fi

   if [ "$vm" = "" ] ; then
      echo "$cmdname: agent start: JRE is not configured"
      exit 1
   fi

   # if PID file exists check to see if the agent is running   
   if [ -r "$pidfile" ] ; then
      agent_pid
      agent_stat "up"
      if [ $result = $TRUE ] ; then
         echo "$cmdname: agent start: SAS Deployment Agent is already running. Process Identifier (PID) is $pid"
         exit 1
      fi  
   fi

   cd "$agentdir"
   launcher=`ls "plugins/"org.eclipse.equinox.launcher_*.jar`
   platform=`uname`
   if [ $platform = HP-UX -o $platform = SunOS ] ; then
      d64arg="-d64"
   else
      d64arg=""
   fi
   filter_opts="-Djdk.serialFilter=java.util.*;java.lang.*;javax.management.remote.message.*;javax.management.ObjectName;!sun.rmi.server.ActivationGroupImpl;!*"
   "$vm" $d64arg -Xmx512m $filter_opts -Declipse.ignoreApp=true -Dsolstice.log.filename=agent_${hostname} -Dosgi.logfile=logs/osgi_${hostname}.log -Dsolstice.input.properties=config/agent.properties -cp "$launcher" org.eclipse.equinox.launcher.Main -configuration configurations/agent -noExit 1>/dev/null 2>&1 & 
   cd "$pwd"
   echo $! >"$pidfile"
   echo "$cmdname: agent start: SAS Deployment Agent started."
   exit 0
}


#
# set the Java runtime to use
#
setjre () {
   if [ "$1" = "" ] ; then
      echo location required
      exit 1
   fi

   location=$1

   shift
   if [ "$1" != "" ] ; then
      echo "unrecognized parameter ($1)"
      exit 1
   fi

   set_vm "$location"

   exit 0
}

#
# Load data up from a configuration file
#
load_data () {
   file=$1

   if [ "`uname`" = "OS/390" ] ; then
      line=`iconv -f utf-8 -t ibm-1047 "$file" | grep "^wrapper.java.command="`
   else
      line=`grep "^wrapper.java.command=" "$file"`
   fi

   set_vm "`get_property_value "$line"`"
}

#
# Get a property key
#
get_property_key () {
   line=`echo $1 | tr -d '\15\32'`

   if [ "`echo $line | sed -n '/^[^=]*[^=]=.*$/ {p;}'`" ] ; then
      echo $line | sed 's/^\([^=]*[^=]\)=\(.*\)$/\1/'
   fi
}

#
# Get a property value
#
get_property_value () {
   line=`echo $1 | tr -d '\15\32'`

   if [ "`echo $line | sed -n '/^[^=]*[^=]=.*$/ {p;}'`" ] ; then
      echo $line | sed 's/^\([^=]*[^=]\)=\(.*\)$/\2/'
   fi
}

ask_vm () {
   if [ "$vm" = "" ] ; then
      echo
      echo 'Enter the path to an installed Java 1.7.'
      echo "To exit this utility, type 'Q' followed by the Enter key."
      echo
      echo '   For example:'
      echo '      /usr/bin/java'

      while [ "$vm" = "" ] ; do
         echo
         echo ">>>"
         read input

         if [ "$input" = "q" -o "$input" = "Q" ] ; then
            exit 1
         fi

         set_vm "$input"
      done
   fi
}

#
# Parse the cmd line arguments if no arg is passed drop into
# interactive mode and ask for a vm if one isn't set.
#
parse_mode () {
   case $1 in
      start)
         shift
         agent_start "$@"
         ;;
      stop)
         shift
         agent_stop "$@"
         ;; 
      stat)
         shift
         # initialize pid var or exit with an error
         # We can't stat an agent without a PID
         read_pid 
         # call stat function with remaining args and -v flag
         agent_stat "$@" "-v"
         ;;
      setjre)
         shift
         setjre "$@"
         ;;
      *)
         ask_vm
         ;;
   esac

   return
}

#####
# Main body of script...
#####
load_data "$agentdir/config/wrapper.conf"

parse_mode "$@"

exit "$rc"
